"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.requestFromGraphqlDocument = requestFromGraphqlDocument;
exports.requestFromGraphqlAst = requestFromGraphqlAst;
exports.createRequest = createRequest;
exports.requestedFieldKey = requestedFieldKey;

var _lodash = require("lodash");

var _values = require("graphql/execution/values");

var _directives = require("graphql/type/directives");

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function requestFromGraphqlDocument(document, root, variables) {
    function definitionsOfKind(kind) {
        return document.definitions.filter(function (definition) {
            return definition.kind === kind;
        });
    }

    var fragments = (0, _lodash.keyBy)(definitionsOfKind("FragmentDefinition"), function (definition) {
        return definition.name.value;
    });

    var query = single(definitionsOfKind("OperationDefinition"));

    return requestFromGraphqlAst(query, root, null, variables, fragments);
}

function requestFromGraphqlAst(ast, root, field, variables, fragments) {
    return reader(variables, fragments)(ast, root, field);
}

function reader(variables, fragments) {
    function readAst(ast, root, field) {
        var isField = ast.kind === "Field";
        return createRequest({
            field: field,
            key: isField ? requestedFieldKey(ast) : null,
            args: graphqlArgs(ast, field),
            selections: graphqlSelections(ast, root)
        });
    }

    function graphqlArgs(ast, field) {
        if (field) {
            var argumentDefinitions = (0, _lodash.map)(field.args, function (arg, argName) {
                return {
                    name: argName,
                    description: arg.description === undefined ? null : arg.description,
                    type: arg.type,
                    defaultValue: arg.defaultValue === undefined ? null : arg.defaultValue
                };
            });
            return (0, _values.getArgumentValues)(argumentDefinitions, ast.arguments, variables);
        } else {
            return {};
        }
    }

    function graphqlSelections(ast, root) {
        if (ast.selectionSet) {
            var _ret = function () {
                var fields = root.fields();
                var fieldSelections = mergeFields(collectFields(ast));

                return {
                    v: fieldSelections.map(function (selection) {
                        var field = fields[requestedFieldName(selection)];
                        return readAst(selection, field._target, field);
                    })
                };
            }();

            if ((typeof _ret === "undefined" ? "undefined" : _typeof(_ret)) === "object") return _ret.v;
        } else {
            return [];
        }
    }

    function collectFields(ast) {
        var fieldSelections = [];
        addFields(ast, fieldSelections);
        return fieldSelections;
    }

    function addFields(ast, fieldSelections) {
        ast.selectionSet.selections.filter(shouldIncludeNode).forEach(function (selection) {
            if (selection.kind === "Field") {
                fieldSelections.push(selection);
            } else if (selection.kind === "FragmentSpread") {
                // TODO: handle type conditions
                addFields(fragments[selection.name.value], fieldSelections);
            } else if (selection.kind === "InlineFragment") {
                addFields(selection, fieldSelections);
            } else {
                throw new Error("Unknown selection: " + selection.kind);
            }
        });
    }

    function shouldIncludeNode(node) {
        for (var directiveIndex = 0; directiveIndex < node.directives.length; directiveIndex++) {
            var directive = node.directives[directiveIndex];
            if (directive.name.value === "include") {
                var args = (0, _values.getArgumentValues)(_directives.GraphQLIncludeDirective.args, directive.arguments, variables);
                if (args.if === false) {
                    return false;
                }
            } else if (directive.name.value === "skip") {
                var _args = (0, _values.getArgumentValues)(_directives.GraphQLSkipDirective.args, directive.arguments, variables);
                if (_args.if === true) {
                    return false;
                }
            } else {
                throw new Error("Unknown directive: " + directive.name.value);
            }
        }
        return true;
    }

    function mergeFields(selections) {
        var merged = [];
        var byKey = {};

        selections.forEach(function (selection) {
            var key = requestedFieldKey(selection);
            if (byKey[key]) {
                if (selection.selectionSet) {
                    var _byKey$key$selectionS;

                    (_byKey$key$selectionS = byKey[key].selectionSet.selections).push.apply(_byKey$key$selectionS, _toConsumableArray(selection.selectionSet.selections));
                }
            } else {
                byKey[key] = (0, _lodash.cloneDeep)(selection);
                merged.push(byKey[key]);
            }
        });

        return merged;
    }

    return readAst;
}

function createRequest(request) {
    return _extends({
        field: null,
        args: {},
        selections: [],
        joinSelections: []
    }, request);
}

function requestedFieldName(ast) {
    return ast.name.value;
}

function requestedFieldKey(ast) {
    return ast.alias ? ast.alias.value : requestedFieldName(ast);
}

function single(values) {
    if (values.length === 1) {
        return values[0];
    } else {
        throw new Error("Expected 1 but got " + values.length);
    }
}
//# sourceMappingURL=requests.js.map